package net.geraldloeffler.tasktracker2006.backend.accounting;

import javax.annotation.Resource;
import javax.ejb.Stateless;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.Message;
import javax.jms.MessageProducer;
import javax.jms.Session;

import net.geraldloeffler.tasktracker2006.domain.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Stateless session bean implementation of the interface for a client to the
 * accounting system (via the accounting system gateway)
 * 
 * @author Gerald Loeffler
 */
@Stateless
public class AccountingSystemClientBean implements AccountingSystemClient {
  private static final Log log = LogFactory.getLog(AccountingSystemClientBean.class);
  
  @Resource(mappedName = "queue/tasktracker/accounting")
  private Destination destination;
  
  @Resource(mappedName = "ConnectionFactory")
  private ConnectionFactory connectionFactory;
  
  public void replicateEntryToAccountingSystem(Entry entry) {
    if (entry == null) return;
    
    try {
      log.debug("replicateEntryToAccountingSystem(): creating connection");
      final Connection connection = connectionFactory.createConnection();
      log.debug("replicateEntryToAccountingSystem(): creating session");
      final Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

      log.debug("replicateEntryToAccountingSystem(): creating object message");
      final Message message = session.createObjectMessage(entry);

      log.debug("replicateEntryToAccountingSystem(): creating message producer");
      final MessageProducer messageProducer = session.createProducer(destination);
      log.debug("replicateEntryToAccountingSystem(): sending message");
      messageProducer.send(message);
      log.info("replicateEntryToAccountingSystem(): message successfully sent to the (queue for the) accounting system gateway");
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }
}
