package net.geraldloeffler.tasktracker2006.backend.accounting;

import java.util.Random;

import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;

import net.geraldloeffler.tasktracker2006.domain.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Gateway to the accounting system in the form of a message-driven bean.
 * 
 * @author Gerald Loeffler
 */
@MessageDriven(activationConfig = {
    @ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
    @ActivationConfigProperty(propertyName = "destination", propertyValue = "queue/tasktracker/accounting") })
public class AccountingSystemGW implements MessageListener {
  private static final Log log = LogFactory.getLog(AccountingSystemGW.class);
  
  final Random random = new Random();

  public void onMessage(Message msg) {
    log.debug("onMessage() called");

    if (!(msg instanceof ObjectMessage)) {
      log.warn("onMessage(): unexpected message type received: " + msg.getClass() + " - dropping message!");
      return;
    }
    Object o;
    try {
      o = ((ObjectMessage) msg).getObject();
    } catch (Exception e) {
      log.warn("onMessage(): failed to get object from message: " + e.getMessage() + " - dropping message!");
      return;
    }
    if (o == null || !(o instanceof Entry)) {
      log.warn("onMessage(): unexpected object received in message: " + o + " - dropping message!");
      return;
    }
    final Entry entry = (Entry) o;
    
    sendToAccountingSystem(entry);
    log.info("onMessage(): successfully sent message to accounting system");
  }

  private void sendToAccountingSystem(Entry entry) {
    // simulate sending to account system that fails in 50% percent of cases
    if (random.nextBoolean()) {
      log.warn("sendToAccountingSystem(): sending entry to accounting system failed - will try again");
      throw new RuntimeException("accounting system unavailable");
    } else {
      log.info("sendToAccountingSystem(): successfully sent entry to accounting system");
    }
  }
}
