package net.geraldloeffler.tasktracker2006.dao;

import java.util.Date;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TemporalType;
import net.geraldloeffler.tasktracker2006.domain.Entry;

/**
 * Implementation of Data Access Object (DAO) for Entry persistent domain
 * objects.
 * 
 * @author Gerald Loeffler
 */
@Stateless
public class EntryDAOBean implements EntryDAO {
  @PersistenceContext
  private EntityManager em;

  public Entry merge(Entry entry) {
    if (entry == null) return null;
    return em.merge(entry);
  }
  
  public Entry findByID(int id) {
    return em.find(Entry.class, id);
  }

  public List<Entry> findAll() {
    return em.createNamedQuery("findAllEntries").getResultList();
  }

  public List<Entry> findActiveAtOrAfter(Date dateTime) {
    if (dateTime == null) throw new IllegalArgumentException("dateTime is null");
    return em.createNamedQuery("findEntriesActiveAtOrAfter").setParameter(
        "dateTime", dateTime, TemporalType.TIMESTAMP).getResultList();
  }

  public List<Entry> findActiveAtOrBefore(Date dateTime) {
    if (dateTime == null) throw new IllegalArgumentException("dateTime is null");
    return em.createNamedQuery("findEntriesActiveAtOrBefore").setParameter(
        "dateTime", dateTime, TemporalType.TIMESTAMP).getResultList();
  }

  public List<Entry> findActiveInPeriod(Date fromDateTime, Date toDateTime) {
    if (fromDateTime == null) throw new IllegalArgumentException(
        "fromDateTime is null");
    if (toDateTime == null) throw new IllegalArgumentException(
        "toDateTime is null");
    return em.createNamedQuery("findEntriesActiveInPeriod").setParameter(
        "fromDateTime", fromDateTime, TemporalType.TIMESTAMP).setParameter(
        "toDateTime", toDateTime, TemporalType.TIMESTAMP).getResultList();
  }
}
