package net.geraldloeffler.tasktracker2006.domain;

import java.io.Serializable;
import java.util.Date;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;

/**
 * Entry persistent domain class. An Entry relates to exactly one Task.
 * 
 * @author Gerald Loeffler
 */
@Entity
@NamedQueries( {
    @NamedQuery(name = "findAllEntries", query = "select e from Entry e order by e.from, e.to"),
    @NamedQuery(name = "findEntriesActiveAtOrAfter", query = "select e from Entry e where e.to >= :dateTime order by e.from, e.to"),
    @NamedQuery(name = "findEntriesActiveAtOrBefore", query = "select e from Entry e where e.from <= :dateTime order by e.from, e.to"),
    @NamedQuery(name = "findEntriesActiveInPeriod", query = "select e from Entry e where "
        + "(e.from >= :fromDateTime and e.from <= :toDateTime) or (e.from <= :fromDateTime and e.to >= :fromDateTime) "
        + "order by e.from, e.to") })
public class Entry implements Serializable {
  private static final long serialVersionUID = 1L;
  
  @Id
  @GeneratedValue(strategy = GenerationType.AUTO)
  private Integer id;
  private String  description;
  @Column(name = "from_utc", nullable = false)
  private Date    from;
  @Column(name = "to_utc")
  private Date    to;
  @Column(name = "user_name", nullable = false)
  private String  userName;
  @ManyToOne(optional = false, cascade = { CascadeType.MERGE,
      CascadeType.PERSIST, CascadeType.REFRESH }, fetch = FetchType.EAGER)
  private Task    task;

  /** Required no-arg constructor (but package-scoped) */
  Entry() {}

  /** Minimal constructor */
  public Entry(Task task, Date from, String userName) {
    setTask(task);
    setFrom(from);
    setUserName(userName);
  }

  public Integer getId() {
    return id;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public Date getFrom() {
    return from;
  }

  private void setFrom(Date from) {
    if (from == null) throw new IllegalArgumentException("from is null");
    this.from = from;
  }

  public Date getTo() {
    return to;
  }

  public void setTo(Date to) {
    if (to != null && to.before(from)) throw new IllegalArgumentException(
        "to before from");
    this.to = to;
  }

  public String getUserName() {
    return userName;
  }

  private void setUserName(String userName) {
    if (userName == null) throw new IllegalArgumentException("userName is null");
    this.userName = userName;
  }

  public Task getTask() {
    return task;
  }

  private void setTask(Task task) {
    if (task == null) throw new IllegalArgumentException("task is null");
    this.task = task;
  }

  @Override
  public String toString() {
    return this.getClass().getSimpleName() + "[id=" + getId() + "]";
  }  
}
