package net.geraldloeffler.tasktracker2006.domain;

import java.io.Serializable;
import java.util.LinkedHashSet;
import java.util.Set;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;

/**
 * Project persistent domain class. A Project is composed of an arbitrary number
 * of Tasks.
 * 
 * @author Gerald Loeffler
 */
@Entity
public class Project implements Serializable {
  private static final long serialVersionUID = 1L;
  
  @Id
  @GeneratedValue(strategy = GenerationType.AUTO)
  private Integer   id;
  @Column(unique = true, nullable = false)
  private String    name;
  private String    description;
  @OneToMany(mappedBy = "project", cascade = CascadeType.ALL, fetch = FetchType.EAGER)
  private Set<Task> tasks = new LinkedHashSet<Task>();

  /** required no-arg constructor (but package-scoped) */
  Project() {}

  /** Minimal constructor */
  public Project(String name) {
    setName(name);
  }

  public Integer getId() {
    return id;
  }

  public String getName() {
    return name;
  }

  private void setName(String name) {
    if (name == null) throw new IllegalArgumentException("name is null");
    this.name = name;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public Set<Task> getTasks() {
    return tasks;
  }
  
  @Override
  public String toString() {
    return this.getClass().getSimpleName() + "[id=" + getId() + "]";
  }
}
