package net.geraldloeffler.tasktracker2006.domain;

import java.io.Serializable;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.Table;
import javax.persistence.UniqueConstraint;

/**
 * Task persistent domain class. A Task belongs to exactly one Project.
 * 
 * @author Gerald Loeffler
 */
@Entity
@Table(uniqueConstraints = @UniqueConstraint(columnNames = { "name",
    "project_id" }))
@NamedQueries( { @NamedQuery(name = "findAllTasks", query = "select t from Task t order by t.project.name, t.name") })
public class Task implements Serializable {
  private static final long serialVersionUID = 1L;
  
  @Id
  @GeneratedValue(strategy = GenerationType.AUTO)
  private Integer id;
  @Column(nullable = false)
  private String  name;
  private String  description;
  @ManyToOne(optional = false, cascade = { CascadeType.MERGE,
      CascadeType.PERSIST, CascadeType.REFRESH }, fetch = FetchType.EAGER)
  private Project project;

  /** Required no-arg constructor (but package-scoped) */
  Task() {}

  /** Minimal constructor */
  public Task(Project project, String name) {
    setProject(project);
    setName(name);
  }

  public Integer getId() {
    return id;
  }

  public String getName() {
    return name;
  }

  private void setName(String name) {
    if (name == null) throw new IllegalArgumentException("name is null");
    this.name = name;
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public Project getProject() {
    return project;
  }

  private void setProject(Project project) {
    if (project == null) throw new IllegalArgumentException("project is null");
    
    // manually ensure relational integrity of of this bidirectional
    // relationship
    if (this.project != null && this.project.getTasks() != null) {
      this.project.getTasks().remove(this);
    }
    this.project = project;
    if (this.project.getTasks() != null) {
      this.project.getTasks().add(this);
    }
  }
  
  @Override
  public String toString() {
    return this.getClass().getSimpleName() + "[id=" + getId() + "]";
  }
}
