package net.geraldloeffler.tasktracker2006.service;

import java.security.Principal;

import javax.annotation.Resource;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJBContext;
import javax.ejb.Stateless;

import net.geraldloeffler.tasktracker2006.domain.Entry;

/**
 * Stateless session bean implementation of SecurityRules.
 * 
 * @author Gerald Loeffler
 */
@Stateless
@RolesAllowed("TaskTrackerUser")
public class SecurityRulesBean implements SecurityRules {
  public static final String ADMIN = "TaskTrackerAdmin";
  
  @Resource
  private EJBContext context;
  
  public void checkCreateEntry(Entry entry) {
    // only static membership to role(s) required
  }
  
  public void checkUpdateEntry(Entry entry) {
    if (entry == null) return;
    
    if (isAdmin()) return;
    if (isPrincipal(entry.getUserName())) return;
    
    throw new SecurityException(pname() + " not allowed to update entry " + entry);
  }
  
  private String pname() {
    final Principal p = context.getCallerPrincipal();
    return p == null ? null : p.getName();
  }

  private boolean isAdmin() {
    return context.isCallerInRole(ADMIN);
  }
  
  private boolean isPrincipal(String principalName) { 
    if (principalName == null) {
      return pname() == null;
    }
    return principalName.equals(pname()); 
  }
}
