package net.geraldloeffler.tasktracker2006.webui.controller;

import java.security.Principal;

import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;

/**
 * Base class of all tasttracker controllers in the form of JSF managed beans.
 * 
 * @author Gerald Loeffler
 */
public abstract class Controller {
  protected static final String SUCCESS    = "success";
  protected static final String FAILURE    = "failure";
  protected static final String EDIT_ENTRY = "edit-entry";
  
  /**
   * Determine the name of the currently authenticated principal (user). Since
   * this application may only be used by authenticated users, this method
   * throws an IllegalStateException if there is no authenticated user (as this
   * proves a configuration problem in the application).
   * 
   * @return the name of the Principal representing the currently authenticated
   *         user. Never returns null.
   */
  protected String userName() {
    final Principal principal = externalContext().getUserPrincipal();
    if (principal == null) {
      throw new IllegalStateException("The current user does not seem to be logged-in!");
    }
    
    final String userName = principal.getName();
    if (userName == null) {
      throw new IllegalStateException("The current user does not have a name!");
    }
    
    return userName;
  }

  protected void globalMessage(String message) {
    FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Please note.", message));
  }
  
  protected void exceptionMessage(Exception e) {
    if (e == null) return;
    
    globalMessage(e.getMessage());
  }
  
  protected String requestParam(String name) {
    return (String) externalContext().getRequestParameterMap().get(name);
  }

  protected ExternalContext externalContext() {
    return FacesContext.getCurrentInstance().getExternalContext();
  }
}
