package net.geraldloeffler.tasktracker2006.webui.controller;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.annotation.EJB;
import javax.faces.model.SelectItem;

import net.geraldloeffler.tasktracker2006.domain.Entry;
import net.geraldloeffler.tasktracker2006.domain.Task;
import net.geraldloeffler.tasktracker2006.service.EntryService;
import net.geraldloeffler.tasktracker2006.service.TaskService;
import net.geraldloeffler.tasktracker2006.webui.util.ServiceLookup;

/**
 * Controller for create-entry use case in the form of a JSF managed bean.
 * 
 * @author Gerald Loeffler
 */
public class CreateEntry extends Controller {
  private List<Task>       tasks;
  private List<SelectItem> taskSIs;
  private int              taskValue;
  private Date             from;
  @EJB
  private EntryService     entryService = ServiceLookup.entryService();
  @EJB
  private TaskService      taskService  = ServiceLookup.taskService();
  private EditEntry        editEntry;

  private void reset() {
    taskValue = 0;
    from = null;
  }

  public void setEditEntry(EditEntry editEntry) {
    this.editEntry = editEntry;
  }

  private void initTasks() {
    // get all tasks and convert them to list of SelectItems
    tasks = taskService.getAllTasks();
    taskSIs = new ArrayList<SelectItem>();
    for (int i = 0; i < tasks.size(); i++) {
      final Task task = tasks.get(i);
      // SelectItem value must be a String
      // unless you register a converter with the UISelectOne
      taskSIs.add(new SelectItem(Integer.toString(i), task.getName()));
    }
    taskValue = 0;
  }

  public List<SelectItem> getTaskSIs() {
    if (taskSIs == null) initTasks();
    return taskSIs;
  }

  public String getTaskValue() {
    if (taskSIs == null) initTasks();
    return Integer.toString(taskValue);
  }

  public void setTaskValue(String v) {
    taskValue = Integer.parseInt(v);
  }

  public Date getFrom() {
    // default to current date and time
    if (from == null) from = new Date();
    return from;
  }

  public void setFrom(Date from) {
    this.from = from;
  }

  public String createEntry() {
    final Task task = tasks.get(taskValue);
    if (from == null) {
      globalMessage("'from' is required");
      return null;
    }
    Entry entry;
    try {
      entry = new Entry(task, from, userName());
      // ask service layer to create entry
      entry = entryService.createEntry(entry);
    } catch (Exception e) {
      exceptionMessage(e);
      return null;
    }
    // set to-date to something reasonable for the following user interaction
    entry.setTo(entry.getFrom());
    // pass entry to EditEntry controller
    editEntry.setEntry(entry);
    // reset for next execution of this use-case
    reset();
    // explicitly pass control to edit-entry use case
    return EDIT_ENTRY;
  }
}
