package net.geraldloeffler.tasktracker2006.webui.controller;

import javax.annotation.EJB;

import net.geraldloeffler.tasktracker2006.domain.Entry;
import net.geraldloeffler.tasktracker2006.service.EntryService;
import net.geraldloeffler.tasktracker2006.webui.util.ServiceLookup;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Controller for create-entry use case in the form of a JSF managed bean.
 * 
 * @author Gerald Loeffler
 */
public class EditEntry extends Controller {
  public static final String ENTRY_ID_REQUEST_PARAM_NAME = "entryID";
  
  private static final Log log = LogFactory.getLog(EditEntry.class);
  
  private Entry        entry;
  @EJB
  private EntryService entryService = ServiceLookup.entryService();

  private void reset() {
    entry = null;
  }

  public void setEntry(Entry entry) {
    this.entry = entry;
  }

  public Entry getEntry() {
    initEntryFromRequestParam();
    
    return entry;
  }

  private void initEntryFromRequestParam() {
    final String entryID = requestParam(ENTRY_ID_REQUEST_PARAM_NAME);
    if (entryID == null) return;

    try {
      int id = Integer.parseInt(entryID);

      log.info("initEntryFromRequestParam(): setting Entry with id " + id);
      this.entry = entryService.getEntryByID(id);
      if (this.entry == null) log.warn("initEntryFromRequestParam(): no Entry found with id " + id);
    } catch (Exception e) {
      log.info("initEntryFromRequestParam(): exception getting Entry with id " + entryID);
      exceptionMessage(e);
    }
  }

  public String saveEntry() {
    try {
      entry = entryService.updateEntry(entry);
    } catch (Exception e) {
      exceptionMessage(e);
      return null;
    }
    // reset for next execution of this use-case
    reset();
    
    return SUCCESS;
  }
}
